@@ -1,25 +1,25 @@
 # -*- coding: utf-8 -*-
 
+import colorsys
+import random
+import string
 from collections import namedtuple
-from functools import wraps
-from typing import Callable
+from datetime import datetime
+from datetime import tzinfo
 from enum import IntEnum
+from functools import wraps
 from time import time_ns
-from string import ascii_letters, digits
-from random import choice
-from datetime import (
-    datetime as dt,
-    timezone as tz,
-    timedelta as td
-)
+from typing import Callable
+from typing import Optional
+from typing import Union
 
 __all__ = ('get_timestamp', '_isdecimal', 'rstring',
-           'async_cache', 'TimeScale', 'timef')
+           'async_cache', 'TimeScale', 'timef',
+           'rainbow_color_stops', 'magnitude_fmt_time')
 
 ts_fmt = ('%I:%M:%S%p', '%d/%m/%Y %I:%M:%S%p')
-tz_est = tz(td(hours = -5), 'EST') # TODO: fix for edt switch lol
-def get_timestamp(full: bool = False) -> str:
-    return f'{dt.now(tz = tz_est):{ts_fmt[full]}}'
+def get_timestamp(full: bool = False, tz: Optional[tzinfo] = None) -> str:
+    return f'{datetime.now(tz=tz):{ts_fmt[full]}}'
 
 def _isdecimal(s: str, _float: bool = False,
                _negative: bool = False) -> None:
@@ -31,9 +31,9 @@ def _isdecimal(s: str, _float: bool = False,
 
     return s.isdecimal()
 
-__chars = ascii_letters + digits
+__chars = string.ascii_letters + string.digits
 def rstring(l: int, seq: str = __chars) -> str:
-    return ''.join((choice(seq) for _ in range(l)))
+    return ''.join((random.choice(seq) for _ in range(l)))
 
 _CacheStats = namedtuple('CacheStats', ['hits', 'misses'])
 
@@ -138,3 +138,24 @@ def timef(f: Callable):
     wrapper.calls = calls # maybe temp?
     wrapper.average = average
     return wrapper
+
+def rainbow_color_stops(
+    n: int = 10,
+    lum: float = 0.5,
+    end: float = 2 / 3
+) -> list[tuple[int, int, int]]:
+    # https://stackoverflow.com/a/58811633
+    return [(r * 255, g * 255, b * 255)
+            for r, g, b in [colorsys.hls_to_rgb(end * i / (n - 1), lum, 1)
+                            for i in range(n)]]
+
+# TODO: genericize this to metric all units?
+TIME_ORDER_SUFFIXES = ['nsec', 'μsec', 'msec', 'sec']
+def magnitude_fmt_time(
+    t: Union[int, float] # in nanosec
+) -> str:
+    for suffix in TIME_ORDER_SUFFIXES:
+        if t < 1000:
+            break
+        t /= 1000
+    return f'{t:.2f} {suffix}'
