@@ -1,8 +1,11 @@
 # -*- coding: utf-8 -*-
 
+import functools
+
 __all__ = ('Version',)
 
-class Version: # TODO: why fixed length? also could maybe inherit comp ops
+@functools.total_ordering
+class Version:
     __slots__ = ('major', 'minor', 'micro')
     def __init__(self, major: int, minor: int, micro: int) -> None:
         self.major = major
@@ -10,7 +13,7 @@ class Version: # TODO: why fixed length? also could maybe inherit comp ops
         self.micro = micro
 
     def __repr__(self) -> str:
-        return '{0}.{1}.{2}'.format(*self.as_tuple)
+        return f'{self.major}.{self.minor}.{self.micro}'
 
     def __hash__(self) -> int:
         return self.as_tuple.__hash__()
@@ -18,27 +21,15 @@ class Version: # TODO: why fixed length? also could maybe inherit comp ops
     def __eq__(self, other: 'Version') -> bool:
         return self.as_tuple == other.as_tuple
 
-    def __ne__(self, other: 'Version') -> bool:
-        return self.as_tuple != other.as_tuple
-
     def __lt__(self, other: 'Version') -> bool:
         return self.as_tuple < other.as_tuple
 
-    def __le__(self, other: 'Version') -> bool:
-        return self.as_tuple <= other.as_tuple
-
-    def __gt__(self, other: 'Version') -> bool:
-        return self.as_tuple > other.as_tuple
-
-    def __ge__(self, other: 'Version') -> bool:
-        return self.as_tuple >= other.as_tuple
-
     @property
     def as_tuple(self) -> tuple[int]:
         return (self.major, self.minor, self.micro)
 
     @classmethod
-    def from_str(cls, s: str):
+    def from_str(cls, s: str) -> 'Version':
         if len(split := s.split('.')) != 3:
             return
 
